# -*- coding: utf-8 -*-
#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
from arcpy.arcobjects._base import _ObjectWithoutInitCall, passthrough_attr, pass_parameterized_attr
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
from arcpy.geoprocessing._base import gp_fixargs
import codecs


class Chart(_ObjectWithoutInitCall):
    """The Chart class defines an ArcGIS Pro chart. The class allows you to create different types of charts, including
    bar charts, line charts, histograms, and scatter plots. Also, you can use the class to define the chart title, axes,
    and other properties.
    """
    _hId = passthrough_attr('horizontalAxisId')
    _vId = passthrough_attr('verticalAxisId')
    _chartType = passthrough_attr('chartType')
    title = passthrough_attr('title')
    description = passthrough_attr('description')
    color = passthrough_attr('color')
    dataSource = passthrough_attr('dataSource')

    class _arc_base(object):
        def __init__(self, parent):
            self._parent = parent
            self._arc_object = parent._arc_object

    class _seriesType(_arc_base):
        pass

        def __init__(self, parent):
            super().__init__(parent)

        def __repr__(self):
            return 'Chart("{} series")'.format(self._parent._chartType)

        def __str__(self):
            return 'Chart("{} series")'.format(self._parent._chartType)

    class _axis(_arc_base):
        """Sets additional properties that apply to axis only.
        field
            The name of the attribute field to display on the axis.

        sort
            The sorting method applied to this axis. ASC for ascending sorting and DESC for descending sorting. This
            only applies to bar charts.

        title
            The label of the axis that displays on the chart.
        """
        _axisType = ''

        class _guide(object):
            def __init__(self, _name):
                self.name = _name
                self.valueFrom = None
                self.valueTo = None
                self.label = None
        
        class _Guides(object):
            def new(self, name, valueFrom, valueTo = None, label = None):
                guide = self._axis._guide(name)
                guide.valueFrom = valueFrom
                guide.valueTo = valueTo
                guide.label = label
                
                guides = self._axis._guides
                if guides == None:
                    guides = []

                guides.append(self._getGuideDict(guide))

                self._axis._guides = guides
                return guide

            def get(self, index):
                return self._makeGuide(self._axis._guides[index])

            def update(self, guide):
                guides = self._axis._guides
                for i, g in enumerate(guides):
                    if g['name'] == guide.name:
                        guides[i] = self._getGuideDict(guide)
                self._axis._guides = guides
                return    

            def remove(self, guide):
                guides = self._axis._guides
                guides = [g for g in guides if g['name'] != guide.name]
                self._axis._guides = guides    
                return

            def removeByIndex(self, index):
                guides = self._axis._guides
                del guides[index]
                self._axis._guides = guides    
                return
            
            def removeAll(self):
                self._axis._guides = []

            def length(self):
                return len(self._axis._guides)

            def _getGuideDict(self, guide):
                _guide = {}
                _guide['name'] = guide.name
                _guide['valueFrom'] = guide.valueFrom
                if guide.valueTo != None:
                    _guide['valueTo'] = guide.valueTo
                if guide.label != None:
                    _guide['label'] = guide.label
                return _guide

            def _makeGuide(self, guideDict):
                guide = self._axis._guide("new-guide")
                if 'name' in guideDict:
                    guide.name = guideDict['name']
                if 'valueFrom' in guideDict:
                    guide.valueFrom = guideDict['valueFrom']
                if 'valueTo' in guideDict:
                    guide.valueTo = guideDict['valueTo']
                if 'label' in guideDict:
                    guide.label = guideDict['label']
                return guide

            def __init__(self, axis):
                self._axis = axis

        _guides = pass_parameterized_attr('axisProperty', axisId="axisId", type='guides')

        title = pass_parameterized_attr('axisProperty', axisId="axisId", type='title')
        field = pass_parameterized_attr('axisProperty', axisId="axisId", type='field')
        sort = pass_parameterized_attr('axisProperty', axisId="axisId", type='sort')
        minimum = pass_parameterized_attr('axisProperty', axisId="axisId", type='minimum')
        maximum = pass_parameterized_attr('axisProperty', axisId="axisId", type='maximum')
        logarithmic = pass_parameterized_attr('axisProperty', axisId="axisId", type='logarithmic')

        def __repr__(self):
            return "Chart.{}axis".format(self._axisType)

        def __str__(self):
            return "Chart<{}>.{}axis<{}>".format(self._parent._chartType, self._axisType, self.title)

        def __init__(self, parent, i):
            super().__init__(parent)
            if i == 0:
                self._axisType = 'X'
            else:
                self._axisType = 'Y'
            self.axisId = i
            self.guides = self._Guides(self)

    class _bar(_seriesType):
        """Sets additional properties that apply to bar charts only.
        aggregation
            The statistical calculation applied to values that occur at the same value along the x-axis.
            Valid statistics include COUNT, SUM, MEAN, and MEDIAN. No aggregation means that each value will be plotted
            on the chart regardless of overlap or recurring values.

        splitCategory
            A second categorical attribute field that adds a separate series or bar for each unique value in the field.

        multiSeriesDisplay
            A type of multiple series bar chart. Valid options include 'sideBySide', 'stacked' and 'stacked100'
        """
        aggregation = passthrough_attr('aggregation')
        splitCategory = passthrough_attr('splitCategory')
        multiSeriesDisplay = passthrough_attr('multiSeriesDisplay')
        rotated = passthrough_attr('rotated')
        def __init__(self, parent):
            super().__init__(parent)

    class _scatterPlot(_seriesType):
        """Sets additional properties that apply to scatter charts only.
        showTrendLine
            Toggles if the trend line is visible in the histogram. True displays the line,
            False hides the line.
        bubbleMinSize
            Number that determines the minimum size of the bubble if scatterplot has an extra yAxis field to represent a bubble size.
        bubbleMaxSize
            Number that determines the maximum size of the bubble if scatterplot has an extra yAxis field to represent a bubble size.
        """
        def __init__(self, parent):
            super().__init__(parent)
        showTrendLine = passthrough_attr('showTrendLine')
        bubbleMinSize =  passthrough_attr('bubbleMinSize')
        bubbleMaxSize =  passthrough_attr('bubbleMaxSize')

    class _qqPlot(_seriesType):
        """Sets additional properties that apply to histograms only.
        showReferenceLine
            Toggles if the reference line is visible in the histogram. True displays the line,
            False hides the line.

        dataTransformationType
             Determines the data transformation type. Valid options include 'none', 'logarithmic' and
             'squareRoot'.
        """
        showReferenceLine = passthrough_attr('showReferenceLine')
        dataTransformationType = passthrough_attr('dataTransformationType')

        def __init__(self, parent):
            super().__init__(parent)

    class _line(_seriesType):
        """Sets additional properties that apply to line charts only.
        aggregation
            The statistical calculation applied to values that occur at the same value along the x-axis.
            Valid statistics include 'COUNT', 'SUM', 'MEAN', and 'MEDIAN'. No aggregation means that each value will be plotted
            on the chart regardless of overlap or recurring values.

        splitCategory
            A second categorical attribute field that adds a separate series or bar for each unique value in the field.

        timeAggregationType
             Determines the time interval alignment. Valid options include equalIntervalsFromStartTime and
             equalIntervalsFromEndTime.

        timeIntervalSize
             A number for the span of time that will be binned or aggregated together. Must be used in conjunction with
             timeIntervalUnit.

        timeIntervalUnits
             The time unit that corresponds to the timeIntervalSize. Valid options include 'SECONDS', 'MINUTES', 'HOURS',
             'DAYS', 'WEEKS', 'MONTHS', 'YEARS'.

        trimInCompleteTimeInterval
             Determines if incomplete time intervals at the start or end of the data span (depending on the
             timeAggregationType) will be removed from the chart. Incomplete intervals on a chart can cause misleading
             results in which the period in question is under or over-reported due to a different amount of time in the
             interval. True indicates to trim these incomplete intervals, while False does not trim incomplete
             intervals.

        nullPolicy
            Determines how null values are treated. Valid options include 'null', 'zero' and 'interpolate'
        """
        aggregation = passthrough_attr('aggregation')
        splitCategory = passthrough_attr('splitCategory')
        timeAggregationType = passthrough_attr('timeAggregationType')
        timeIntervalUnits = passthrough_attr('timeIntervalUnits')
        timeIntervalSize = passthrough_attr('timeIntervalSize')
        trimIncompleteTimeInterval = passthrough_attr('trimIncompleteTimeInterval')
        nullPolicy = passthrough_attr('nullPolicy')
        rotated = passthrough_attr('rotated')

        def __init__(self, parent):
            super().__init__(parent)

    class _histogram(_seriesType):
        """Sets additional properties that apply to histograms only.
        binCount
            Number that determines the number of bins to display in the histogram.

        showMean
            Toggles if the mean statistical line is visible in the histogram. True displays the line, False hides the
            line.

        showMedian
            Toggles if the median statistical line is visible in the histogram. True displays the line, False hides the
            line.

        showStandardDeviation
            Toggles if the standard deviation statistical line is visible in the histogram. True displays the line,
            False hides the line.

        showComparisonDistribution
            Toggles if the comparison distribution statistical line is visible in the histogram. True displays the line,
            False hides the line.

        dataTransformationType
             Determines the data transformation type. Valid options include `none`, `logarithmic` and
             `squareRoot`.
        """
        binCount = passthrough_attr('binCount')
        showMean = passthrough_attr('showMean')
        showMedian = passthrough_attr('showMedian')
        showStandardDeviation = passthrough_attr('showStandardDeviation')
        showComparisonDistribution = passthrough_attr('showComparisonDistribution')
        dataTransformationType = passthrough_attr('dataTransformationType')

        def __init__(self, parent):
            super().__init__(parent)

    class _boxPlot(_seriesType):
        showOutliers = passthrough_attr('showOutliers')
        standardizeValues = passthrough_attr('standardizeValues')
        splitCategory = passthrough_attr('splitCategory')
        splitCategoryAsMeanLine = passthrough_attr('splitCategoryAsMeanLine')
        rotated = passthrough_attr('rotated')

        def __init__(self, parent):
            super().__init__(parent)

    class _dataClock(_seriesType):
        """Sets additional properties that apply to line charts only.
        aggregation
            The statistical calculation applied to values that occur at the same value along the x-axis.
            Valid statistics include 'COUNT', 'SUM', 'MEAN', and 'MEDIAN'.

        timeRingWedgeUnits
             The description for time unit pair supported in data clock chart. Valid options include
             'YEARS_MONTHS', 'YEARS_WEEKS', 'YEARS_DAYS', 'WEEKS_DAYS', 'DAYS_HOURS'

        nullPolicy
            Determines how null values are treated. Valid options include 'null' and 'zero'

        classificationMethod    
            Valid options include 'equalIntervals', 'geometricalIntervals', 'naturalBreaks', 'quantiles'
        """
        aggregation = passthrough_attr('aggregation')
        timeUnitsRingWedge = passthrough_attr('timeUnitsRingWedge')
        nullPolicy = passthrough_attr('nullPolicy')
        classificationMethod = passthrough_attr('classificationMethod')
        classCount = passthrough_attr('classCount')

        def __init__(self, parent):
            super().__init__(parent)
    
    class _calendarHeatChart(_seriesType):
        """Sets additional properties that apply to line charts only.
        aggregation
            The statistical calculation applied to values that occur at the same value along the x-axis.
            Valid statistics include 'COUNT', 'SUM', 'MEAN', and 'MEDIAN'.

        calendarType
             Supported calendar grid types. Valid options include
             'YEAR_MONTHSDAYS', 'WEEK_DAYSHOURS'

        nullPolicy
            Determines how null values are treated. Valid options include 'null' and 'zero'

        classificationMethod    
            Valid options include 'equalIntervals', 'geometricalIntervals', 'naturalBreaks', 'quantiles'
        """
        aggregation = passthrough_attr('aggregation')
        calendarType = passthrough_attr('calendarType')
        nullPolicy = passthrough_attr('nullPolicy')
        classificationMethod = passthrough_attr('classificationMethod')
        classCount = passthrough_attr('classCount')

        def __init__(self, parent):
            super().__init__(parent)

    class _scatterPlotMatrix(_seriesType):
        """Sets additional properties that apply to scatter charts only.
        showTrendLine
            Toggles if the trend line is visible in the histogram. True displays the line,
            False hides the line.
        """
        def __init__(self, parent):
            super().__init__(parent)
        fields = passthrough_attr('fields')
        showTrendLine = passthrough_attr('showTrendLine')
        showHistograms = passthrough_attr('showHistograms')
        showAsRSquared = passthrough_attr('showAsRSquared')


    class _legend(_arc_base):
        """Sets properties of the chart legend.
        visible
            Toggles if the legend is visible in the chart view. True displays the legend, False hides the legend.
        """
        visible = passthrough_attr('legendVisible')

        def __init__(self, parent):
            super().__init__(parent)

    @property
    def type(self):
        return self._chartType

    @type.setter
    def type(self, val):
        """Chart.type{String}
        Sets the type of chart to create. Valid options include.
            bar
            line
            scatter
            histogram
            boxPlot
            qqPlot
            dataClock
            scatterMatrix
            calendarHeatChart
        """
        if getattr(self, '_chartType'):
            raise AttributeError('Cannot change chart type: "{}"'.format(self._chartType))
        else:
            self._createChartType(val)

    @property
    def legend(self):
        return self._create_legend()

    def _create_legend(self):
        if hasattr(self, 'chartLegend'):
            return self.chartLegend
        else:
            setattr(self, 'chartLegend', self._legend(self))
            return self.chartLegend

    def _createChartType(self, val):
        if isinstance(val, str) == False :
            raise RuntimeError("Chart type must be a string")

        val = val.lower()
        if val == 'bar':
            self._create_bar()
        elif val == 'scatter':
            self._create_scatterPlot()
        elif val == 'line':
            self._create_line()
        elif val == 'histogram':
            self._create_histogram()
        elif val == 'boxplot':
            self._create_boxPlot()
        elif val == 'qqplot':
            self._create_qqPlot()
        elif val == 'dataclock':
            self._create_dataClock()
        elif val == 'scattermatrix':
            self._create_scatterPlotMatrix()
        elif val == 'calendarheatchart':
            self._create_calendarHeatChart()
        else:
            raise RuntimeError("Invalid chart type. Valid values: 'bar' | 'histogram' | 'line' | 'scatter' | 'scatterMatrix' | 'boxPlot' | 'qqPlot' | 'dataClock' | 'calendarHeat'")

    def _create_axis(self, i):
        return self._axis(self, i)

    def _create_bar(self):
        setattr(self, '_chartType', 'bar')
        setattr(self, 'bar', self._bar(self))
        return self

    def _create_scatterPlot(self):
        setattr(self, '_chartType', 'scatter')
        setattr(self, 'scatter', self._scatterPlot(self))
        return self

    def _create_line(self):
        setattr(self, '_chartType', 'line')
        setattr(self, 'line', self._line(self))
        return self

    def _create_histogram(self):
        setattr(self, '_chartType', 'histogram')
        setattr(self, 'histogram', self._histogram(self))
        return self

    def _create_boxPlot(self):
        setattr(self, '_chartType', 'boxPlot')
        setattr(self, 'boxPlot', self._boxPlot(self))
        return self

    def _create_qqPlot(self):
        setattr(self, '_chartType', 'qqPlot')
        setattr(self, 'qqPlot', self._qqPlot(self))
        return self

    def _create_dataClock(self):
        setattr(self, '_chartType', 'dataClock')
        setattr(self, 'dataClock', self._dataClock(self))
        return self

    def _create_calendarHeatChart(self):
        setattr(self, '_chartType', 'calendarHeatChart')
        setattr(self, 'calendarHeatChart', self._calendarHeatChart(self))
        return self

    def _create_scatterPlotMatrix(self):
        setattr(self, '_chartType', 'scatterMatrix')
        setattr(self, 'scatterMatrix', self._scatterPlotMatrix(self))
        return self

    def _repr_svg_(self):
        return self.getSVG(400, 300)

    def getSVG(self, width, height):
        return convertArcObjectToPythonObject(self._arc_object.getSVG(width, height))

    def exportToSVG(self, path, width, height):
        file = codecs.open(path, "w", "utf-8")
        file.write(self.getSVG(width, height))
        file.close()

    def addToLayer(self, layer_or_layerfile):
        """Chart.addToLayer(layer_or_layerfile)
        Adds the chart class to a layer.

        layer_or_layerfile{object}
            The chart will be added to this target layer object.

        The chart object properties will be set into the layer definition.
        """
        return convertArcObjectToPythonObject(self._arc_object.addToLayer(*gp_fixargs((layer_or_layerfile,), True)))

    def updateChart(self):
        """Chart.updateChart()
        Updates chart properties to sync changes between the class and the chart associated with a layer.

        After defining chart properties in the chart class, the final step is typically to add the chart to a layer.
        Use the addToLayer method to accomplish this. However, you may wish to further modify the chart properties.
        Instead of starting from scratch with a new chart, you can modify the properties of the original chart class,
        then use the updateChart method to sync any changes into the layer. This will allow the changes you make to be
        presented in the Chart properties pane and chart view.Use updateChart to sync chart property changes into a
        layer.
        """
        return convertArcObjectToPythonObject(self._arc_object.updateChart(*gp_fixargs((), True)))

    def __init__(self, name):
        import arcgisscripting
        self._arc_object = arcgisscripting._chart.Chart(*gp_fixargs((name,), True))
        self.xAxis = self._create_axis(self._hId)
        self.yAxis = self._create_axis(self._vId)

ob_conversion = (
    ('ChartObject', Chart),
)
