# -*- coding: utf-8 -*-
#COPYRIGHT 2020 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""Toolbox Indoors Tools"""
__all__ = ['ClassifyIndoorPathways', 'ConfigureIndoorPositioning',
           'CreateIndoorsDatabase', 'GenerateFacilityEntryways',
           'GenerateFloorTransitions', 'GenerateIndoorPathways',
           'GenerateOccupantFeatures', 'ImportFloorplansToIndoorsGDB',
           'ThinIndoorPathways']
__alias__ = 'indoors'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptooldoc('ConfigureIndoorPositioning_indoors', None)
def ConfigureIndoorPositioning(in_geodatabase=None, encryption_key=None, api_key=None, building_id=None):
    """ConfigureIndoorPositioning_indoors(in_geodatabase, encryption_key, api_key, building_id)

        Writes indoor positioning system configuration information to an
        ArcGIS Indoors geodatabase. The values are used by ArcGIS Indoors for
        iOS and ArcGIS Indoors for Android.The tool is only necessary if your
        organization is using the Indoo.rs
        positioning system.

     INPUTS:
      in_geodatabase (Workspace):
          The Indoors file or enterprise geodatabase for which IPS configuration
          information will be generated.
      encryption_key (String):
          The key used by the tool and Indoors mobile apps to encrypt or
          unencrypt the API Key parameter (api_key in Python) value.
      api_key (String):
          A unique value in the form of a GUID used by Indoors mobile apps to
          enable Indoo.rs indoor positioning. The API key is provided by
          Indoo.rs.
      building_id (String):
          A unique alphanumerical value used by Indoors mobile apps to link the
          site in the mobile map package to the Indoo.rs indoor positioning
          survey. The building ID is provided by Indoo.rs."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ConfigureIndoorPositioning_indoors(*gp_fixargs((in_geodatabase, encryption_key, api_key, building_id), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateIndoorsDatabase_indoors', None)
def CreateIndoorsDatabase(target_gdb=None):
    """CreateIndoorsDatabase_indoors(target_gdb)

        Adds the necessary datasets, feature classes, tables, and
        configurations to a geodatabase to host ArcGIS Indoors data.

     INPUTS:
      target_gdb (Workspace):
          A geodatabase to contain the ArcGIS Indoors Information Model to
          manage indoor GIS information for use with Indoors apps."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateIndoorsDatabase_indoors(*gp_fixargs((target_gdb,), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateOccupantFeatures_indoors', None)
def GenerateOccupantFeatures(in_unit_features=None, unit_id_field=None, in_occupant_table=None, occupant_id_field=None, out_occupant_feature_class=None):
    """GenerateOccupantFeatures_indoors(in_unit_features, unit_id_field, in_occupant_table, occupant_id_field, out_occupant_feature_class)

        Creates or updates employee or occupant point data that conforms to
        the ArcGIS Indoors Information Model.This tool generates point
        features based on input building spaces
        (units), then enriches those points with occupant data. The output of
        this tool can be used to support Indoors mapping and routing.

     INPUTS:
      in_unit_features (Feature Layer):
          The input polygon features representing building spaces that may be
          occupied. In the Indoors Model, this will be the Units layer. The
          centroid of each space will be used as the point location of the
          occupant or occupants.
      unit_id_field (Field):
          The field in the Input Unit Features that will be used as the primary
          key to associate building spaces with records in the Input Occupant
          Table.
      in_occupant_table (Table View):
          The input table that contains information about building occupants.The
          information can be stored in a geodatabase table, a sheet in an
          Excel workbook (.xls or .xlsx file), or a .csv file.
      occupant_id_field (Field):
          The field in the Input Occupant Table that will be used as the primary
          key to associate occupants with Input Unit Features.

     OUTPUTS:
      out_occupant_feature_class (Feature Class):
          The output feature class created from joining the Input Unit Features
          and Input Occupant Table parameter values."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateOccupantFeatures_indoors(*gp_fixargs((in_unit_features, unit_id_field, in_occupant_table, occupant_id_field, out_occupant_feature_class), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ImportFloorplansToIndoorsGDB_indoors', None)
def ImportFloorplansToIndoorsGDB(in_geodatabase=None, in_excel_template=None, uniqueid_delimiter=None, sliver_threshold=None, door_close_buffer=None):
    """ImportFloorplansToIndoorsGDB_indoors(in_geodatabase, in_excel_template, uniqueid_delimiter, {sliver_threshold}, {door_close_buffer})

        Imports floor plans from CAD files into a workspace that contains the
        ArcGIS Indoors Information Model.

     INPUTS:
      in_geodatabase (Workspace):
          The Indoors geodatabase (file or enterprise) into which the floor plan
          data will be loaded.
      in_excel_template (File):
          An Excel spreadsheet (.xls or .xlsx file) that contains input and
          configuration parameters.
      uniqueid_delimiter (String):
          Specifies the delimiter that will separate key values in the Indoors
          model hierarchy.

          * PERIOD-The ID will include key values separated by periods. This is
          default.

          * HYPHEN-The ID will include key values separated by hyphens.

          * UNDERSCORE-The ID will include key values separated by underscores.
      sliver_threshold {Long}:
          The ratio of perimeter to area that defines a sliver polygon. It is
          used when importing Unit polygons to improve the quality of the
          imported data. Unit polygons that are determined to be slivers are
          placed in a review geodatabase located in the scratch folder of the
          ArcGIS Pro project. The default value is 2.
      door_close_buffer {Double}:
          The distance, in inches, the tool will search from a door to find and
          snap to the nearest wall. This parameter is used when the CLOSE_DOORS
          option is set to Y in the input Excel template file. The default value
          is 0."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ImportFloorplansToIndoorsGDB_indoors(*gp_fixargs((in_geodatabase, in_excel_template, uniqueid_delimiter, sliver_threshold, door_close_buffer), True)))
        return retval
    except Exception as e:
        raise e


# Indoors Network toolset
@gptooldoc('ClassifyIndoorPathways_indoors', None)
def ClassifyIndoorPathways(in_unit_features=None, target_pathways=None):
    """ClassifyIndoorPathways_indoors(in_unit_features, target_pathways)

        Classifies pathways as primary or secondary for routing
        priority.Pathways within the selected space (unit) features are ranked
        as
        secondary, with the remaining ranked as primary. These pathways are
        used to generate a network for routing. Secondary pathways in the
        network are low priority pathways and are only used for routing when
        needed to reach a destination.

     INPUTS:
      in_unit_features (Feature Layer):
          The input polygon features representing spaces within a building for
          which Target Pathways are classified. In the ArcGIS Indoors Model,
          this will be the Units layer.
      target_pathways (Feature Layer):
          The existing feature class or feature layer in which pathways will be
          updated. In the ArcGIS Indoors Model, this is the Pathways layer."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ClassifyIndoorPathways_indoors(*gp_fixargs((in_unit_features, target_pathways), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateFacilityEntryways_indoors', None)
def GenerateFacilityEntryways(in_level_features=None, in_unit_features=None, in_door_features=None, target_entryways=None, buffer_size=None, entryway_use_type=None, exterior_unit_exp=None, delete_existing_entryways=None):
    """GenerateFacilityEntryways_indoors(in_level_features, in_unit_features, in_door_features, target_entryways, {buffer_size}, {entryway_use_type}, {exterior_unit_exp}, {delete_existing_entryways})

        Creates or updates points representing a building's entry or exit
        locations.This tool analyzes input unit features to identify the
        exterior edges
        of a building (facility) and generates point features for doors
        located in proximity to those edges. The ArcGIS Indoors network uses
        these entryway points when routing into, out of, and between
        buildings.

     INPUTS:
      in_level_features (Feature Layer):
          The input polygon features representing a floor or floors in one or
          more buildings. In the Indoors Model, this will be the Levels layer.
          The tool processes only the building floors represented by these
          features.
      in_unit_features (Feature Layer):
          The input polygon features representing building spaces. In the
          Indoors Model, this will be the Units layer. The tool uses these
          features when identifying exterior edges of a building.
      in_door_features (Feature Layer):
          The input polyline features representing doors. In the Indoors
          Model, this will be a subset of features from the Details layer. The
          tool uses these features when identifying entryways along the exterior
          of a building. If the source feature class also contains
          features representing other
          architectural details such as windows or stairs, you must first select
          the features in the layer that represent doors. You can use the Select
          Layer By Attribute tool to make a selection.
      target_entryways (Feature Layer):
          The feature class or feature layer to which generated entryway points
          will be written. In the Indoors Model, this will be the
          PointsOfInterest layer.
      buffer_size {Double}:
          The distance, in meters, the tool will search inward and outward from
          a building's exterior edge to identify potential entryways. The
          default value is 0.5 and can range from 0 to 10.
      entryway_use_type {String}:
          The value used to calculate the USE_TYPE field for new entryway
          points. The default value is Entry.
      exterior_unit_exp {SQL Expression}:
          An SQL expression used to define which Input Unit Features represent a
          building's exterior spaces, such as patios or fire escapes. Spaces
          matching this expression will be treated as exterior during entryway
          generation.
      delete_existing_entryways {Boolean}:
          Specifies whether existing entryway features with a USE_TYPE field
          value matching the entryway_use_type parameter value will be deleted
          before creating new entryway points. When deleting existing entryways,
          the tool only identifies entryways on floors included in the
          in_level_features parameter.

          * DELETE_FEATURES-Existing features will be deleted.

          * NO_DELETE_FEATURES-Existing features will not be deleted. This is
          the default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateFacilityEntryways_indoors(*gp_fixargs((in_level_features, in_unit_features, in_door_features, target_entryways, buffer_size, entryway_use_type, exterior_unit_exp, delete_existing_entryways), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateFloorTransitions_indoors', None)
def GenerateFloorTransitions(facility_features=None, transition_unit_features=None, pathway_features=None, target_transitions=None, elevator_delay=None, delete_existing_transitions=None, stairway_unit_exp=None, elevator_unit_exp=None):
    """GenerateFloorTransitions_indoors(facility_features, transition_unit_features, pathway_features, target_transitions, {elevator_delay}, {delete_existing_transitions}, {stairway_unit_exp}, {elevator_unit_exp})

        Creates or updates transition line features that connect floors
        vertically.Selected space (unit) polygons are used to generate a
        vertical line
        between building floors (levels). These transition features are used
        in conjunction with pathway features to generate a network for
        routing.

     INPUTS:
      facility_features (Feature Layer):
          The input polygon features representing a building or buildings. In
          the ArcGIS Indoors Model, this is the Facilities layer. The tool
          processes only the buildings represented by these features.
      transition_unit_features (Feature Layer):
          The input polygon features representing the transition spaces in a
          building. In the ArcGIS Indoors Model, this is the Units layer.
      pathway_features (Feature Layer):
          The input polyline features representing preliminary pathways. The new
          transition features will snap to these polyline features. In the
          Indoors Model, this will be the PrelimPathways layer.
      target_transitions (Feature Layer):
          An existing feature class or layer that is updated with the new
          transitions. In the ArcGIS Indoors Model, this is the
          PrelimTransitions layer.
      elevator_delay {Long}:
          The average elevator transit time. It is one-half the time in seconds
          that an elevator passenger can expect to spend waiting to enter and
          exit the elevator. Using this parameter can improve routing and
          transit time calculations. The value must be equal to or greater than
          zero.
      delete_existing_transitions {Boolean}:
          Specifies whether existing transition features in selected transition
          spaces will be deleted before creating new transition features. If
          this parameter is not used, updated_transitions will include both
          existing and newly created transition features.

          * DELETE_FEATURES-Existing transition features will be deleted. This
          is the default.

          * NO_DELETE_FEATURES-Existing transition features will not be deleted.
      stairway_unit_exp {SQL Expression}:
          An SQL expression used to define which Transition Unit Features
          represent step-based transitions, such as stairs and escalators.
      elevator_unit_exp {SQL Expression}:
          An SQL expression used to define which Transition Unit Features
          represent lift-based transitions, such as elevators."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateFloorTransitions_indoors(*gp_fixargs((facility_features, transition_unit_features, pathway_features, target_transitions, elevator_delay, delete_existing_transitions, stairway_unit_exp, elevator_unit_exp), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateIndoorPathways_indoors', None)
def GenerateIndoorPathways(in_level_features=None, in_detail_features=None, target_pathways=None, lattice_rotation=None, lattice_density=None, restricted_unit_features=None, restricted_unit_exp=None, detail_exp=None):
    """GenerateIndoorPathways_indoors(in_level_features, in_detail_features, target_pathways, {lattice_rotation}, {lattice_density}, {restricted_unit_features}, {restricted_unit_exp}, {detail_exp})

        Generates preliminary pathways that are cut according to obstructions,
        such as walls or columns, on selected floors in one or more buildings
        (facilities). Once stair or elevator floor transitions are added, the
        Thin Indoor Pathways tool is used to create the final indoor network
        dataset.

     INPUTS:
      in_level_features (Feature Layer):
          The input polygon features representing building floors. In the
          Indoors Model, this will be the Levels layer. The tool honors
          selections and definition queries applied to the layer.
      in_detail_features (Feature Layer):
          The input polyline features representing architectural details that
          may serve as barriers to travel within a building. In the Indoors
          Model, this will be the Details layer
      target_pathways (Feature Layer):
          The Indoors geodatabase (file or enterprise) feature class or feature
          layer to which generated pathway polylines will be written. In the
          Indoors Model, this will be the PrelimPathways layer.
      lattice_rotation {Double}:
          The number of degrees by which the input floors' primary travel
          direction is rotated clockwise from due west. If left blank, the tool
          will calculate a value based on the minimum bounding rectangle of each
          floor.The value must be between 0.0 and 180.0.
      lattice_density {Double}:
          The longest distance, in meters, allowed between nodes in the
          generated lattice of pathways. The default value is 0.6.The value must
          be between 0.25 and 0.9.
      restricted_unit_features {Feature Layer}:
          The input polygon features representing restricted and unrestricted
          spaces within a building. In the Indoors Model, this will be the Units
          layer.
      restricted_unit_exp {SQL Expression}:
          An SQL expression used to select Restricted Unit Features in which the
          tool should not generate pathways.
      detail_exp {SQL Expression}:
          An SQL expression used to select Input Detail Features across which
          the tool should not generate pathways."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateIndoorPathways_indoors(*gp_fixargs((in_level_features, in_detail_features, target_pathways, lattice_rotation, lattice_density, restricted_unit_features, restricted_unit_exp, detail_exp), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ThinIndoorPathways_indoors', None)
def ThinIndoorPathways(in_level_features=None, in_pathway_features=None, in_transition_features=None, routable_locations=None, target_pathways=None, target_transitions=None, search_tolerance=None, neighbor_solve_count=None):
    """ThinIndoorPathways_indoors(in_level_features, in_pathway_features, in_transition_features, routable_locations;routable_locations..., target_pathways, target_transitions, {search_tolerance}, {neighbor_solve_count})

        Removes preliminary network pathways that are not needed for routing
        between selected locations on each floor, reducing the network dataset
        size and improving its route-solving performance.

     INPUTS:
      in_level_features (Feature Layer):
          The input polygon features representing a floor or floors in one or
          more buildings (facilities). In the Indoors Model, this will be the
          Levels layer. The tool processes only the floors (levels) represented
          by these features.
      in_pathway_features (Feature Layer):
          The input polyline features representing the preliminary pathways to
          be thinned. In the Indoors Model, this will be the PrelimPathways
          layer.
      in_transition_features (Feature Layer):
          The input polyline features representing the preliminary transitions
          to be thinned. In the Indoors Model, this will be the
          PrelimTransitions layer.
      routable_locations (Feature Layer):
          The input point features representing the locations used to calculate
          routes. In the Indoors Model, this will be the PointsOfInterest layer.
      target_pathways (Feature Layer):
          The existing feature class or feature layer to which the thinned
          pathways will be added. In the Indoors Model, this will be the
          Pathways layer.
      target_transitions (Feature Layer):
          The existing feature class or feature to which thinned transitions
          will be added. In the Indoors Model, this will be the Transitions
          layer.
      search_tolerance {Long}:
          The distance, in meters, the tool will search to find Routable
          Locations features near the input pathways. Routable Locations
          features farther away than this value won't be used for thinning. The
          default value is 5. The value must be 0 or greater.
      neighbor_solve_count {Long}:
          The number of closest neighboring locations to solve when
          calculating routes between a given location and other routable
          locations in the building (facility). The default value is 50.
          The value must be 1 or greater."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ThinIndoorPathways_indoors(*gp_fixargs((in_level_features, in_pathway_features, in_transition_features, routable_locations, target_pathways, target_transitions, search_tolerance, neighbor_solve_count), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject