# -*- coding: utf-8 -*-
#COPYRIGHT 2020 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""Toolbox Space Time Pattern Mining Tools"""
__all__ = ['CreateSpaceTimeCube', 'CreateSpaceTimeCubeDefinedLocations',
           'CreateSpaceTimeCubeMDRasterLayer', 'CurveFitForecast',
           'EmergingHotSpotAnalysis', 'EvaluateForecastsByLocation',
           'ExponentialSmoothingForecast', 'FillMissingValues',
           'ForestBasedForecast', 'LocalOutlierAnalysis',
           'TimeSeriesClustering', 'VisualizeSpaceTimeCube2D',
           'VisualizeSpaceTimeCube3D']
__alias__ = 'stpm'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptooldoc('CreateSpaceTimeCube_stpm', None)
def CreateSpaceTimeCube(in_features=None, output_cube=None, time_field=None, template_cube=None, time_step_interval=None, time_step_alignment=None, reference_time=None, distance_interval=None, summary_fields=None, aggregation_shape_type=None, defined_polygon_locations=None, location_id=None):
    """CreateSpaceTimeCube_stpm(in_features, output_cube, time_field, {template_cube}, {time_step_interval}, {time_step_alignment}, {reference_time}, {distance_interval}, {summary_fields;summary_fields...}, {aggregation_shape_type}, {defined_polygon_locations}, {location_id})

        Summarizes a set of points into a netCDF data structure by aggregating
        them into space-time bins. Within each bin, the points are counted,
        and specified attributes are aggregated. For all bin locations, the
        trend for counts and summary field values are evaluated.

     INPUTS:
      in_features (Feature Layer):
          The input point feature class to be aggregated into space-time bins.
      time_field (Field):
          The field containing the date and time (timestamp) for each point.
          This field must be of type Date.
      template_cube {File}:
          A reference space-time cube used to define the output_cube extent of
          analysis, bin dimensions, and bin alignment. The time_step_interval,
          distance_interval, and reference_time values are also obtained from
          the template cube. This template cube must be a netCDF (.nc) file that
          has been created using this tool.A space-time cube created by
          aggregating into DEFINED_LOCATIONS cannot
          be used as a template_cube.
      time_step_interval {Time Unit}:
          The number of seconds, minutes, hours, days, weeks, or years that will
          represent a single time step. All points within the same
          time_step_interval and distance_interval will be aggregated. (When a
          template_cube is provided, this parameter is ignored, and the
          time_step_interval value is obtained from the template cube). Examples
          of valid entries for this parameter are 1 Weeks, 13 Days, or 1 Months.
      time_step_alignment {String}:
          Defines how aggregation will occur based on a given
          time_step_interval. If a template_cube is provided, the
          time_step_alignment associated with the template_cube overrides this
          parameter setting and the time_step_alignment of the template_cube is
          used.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          START_TIME alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an END_TIME alignment). If the reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an END_TIME alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to use to align the time-step intervals. If you want to
          bin your data weekly from Monday to Sunday, for example, you could set
          a reference time of Sunday at midnight to ensure bins break between
          Sunday and Monday at midnight. (When a template_cube is provided, this
          parameter is ignored and the reference_time is based on the
          template_cube.)
      distance_interval {Linear Unit}:
          The size of the bins used to aggregate the in_features. All points
          that fall within the same distance_interval and time_step_interval
          will be aggregated. When aggregating into a hexagon grid, this
          distance is used as the height to construct the hexagon polygons.
          (When a template_cube is provided, this parameter is ignored and the
          distance interval value will be based on the template_cube.)
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space-time cube. Multiple
          statistic and field combinations can be specified. Null values in any
          of the fields specified will result in that feature being dropped from
          the output cube. If there are null values present in your input
          features, it is highly recommended that you run the Fill Missing
          Values tool before creating a space-time cube. Available
          statistic types are:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field within each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          within each bin.

          * STD-Finds the standard deviation on values in the specified field
          within each bin.

          * MEDIAN-Finds the sorted middle value of all records of the specified
          field within each bin.
                 Available fill types are:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Null values present in any of the summary field records will result in
          those features being excluded from the output cube. If there are null
          values present in your Input Features, it is highly recommended that
          you run the Fill Missing Values tool first. If, after running the Fill
          Missing Values tool, there are still null values present and having
          the count of points in each bin is part of your analysis strategy, you
          may want to consider creating separate cubes, one for the count
          (without Summary Fields) and one for Summary Fields. If the set of
          null values is different for each summary field, you may also consider
          creating a separate cube for each summary field.
      aggregation_shape_type {String}:
          The shape of the polygon mesh into which the input feature point data
          will be aggregated.

          * FISHNET_GRID-The input features will be aggregated into a grid of
          square (fishnet) cells.

          * HEXAGON_GRID-The input features will be aggregated into a grid of
          hexagonal cells.

          * DEFINED_LOCATIONS-The input features will be aggregated into the
          locations provided.
      defined_polygon_locations {Feature Layer}:
          The polygon features into which the input point data will be
          aggregated. These can represent county boundaries, police beats, or
          sales territories for example.
      location_id {Field}:
          The field containing the ID number for each unique location.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created to contain counts and
          summaries of the input feature point data."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCube_stpm(*gp_fixargs((in_features, output_cube, time_field, template_cube, time_step_interval, time_step_alignment, reference_time, distance_interval, summary_fields, aggregation_shape_type, defined_polygon_locations, location_id), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateSpaceTimeCubeDefinedLocations_stpm', None)
def CreateSpaceTimeCubeDefinedLocations(in_features=None, output_cube=None, location_id=None, temporal_aggregation=None, time_field=None, time_step_interval=None, time_step_alignment=None, reference_time=None, variables=None, summary_fields=None, in_related_table=None, related_location_id=None):
    """CreateSpaceTimeCubeDefinedLocations_stpm(in_features, output_cube, location_id, temporal_aggregation, time_field, time_step_interval, {time_step_alignment}, {reference_time}, {variables;variables...}, {summary_fields;summary_fields...}, {in_related_table}, {related_location_id})

        Takes panel data or station data (defined locations where geography
        does not change but attributes are changing over time) and structures
        it into a netCDF data format by creating space-time bins. For all
        locations, the trend for variables or summary fields is evaluated.

     INPUTS:
      in_features (Feature Layer):
          The input point or polygon feature class to be converted into a space-
          time cube.
      location_id (Field):
          An integer field containing the ID number for each unique location.
      temporal_aggregation (Boolean):
          * APPLY_TEMPORAL_AGGREGATION-The space-time cube will temporally
          aggregate your features based on the time_step_interval you provide.
          For example, you have data that has been collected daily and want to
          create a cube with a weekly time_step_interval.

          * NO_TEMPORAL_AGGREGATION-The space-time cube will be created using
          the existing temporal structure of your in_features. For example, you
          have yearly data and want to create a cube with a yearly
          time_step_interval. This is the default.
      time_field (Field):
          The field containing the timestamp for each row in the dataset. This
          field must be of type Date.
      time_step_interval (Time Unit):
          The number of seconds, minutes, hours, days, weeks, or years that will
          represent a single time step. Examples of valid entries for this
          parameter are 1 Weeks, 13 Days, or 1 Months.If temporal_aggregation is
          checked off, you are not aggregating
          temporally, and this parameter should be set to the existing temporal
          structure of your data.If temporal_aggregation is checked on, you are
          aggregating temporally,
          and this parameter should be set to the time_step_interval you want to
          create. All features within the same time_step_interval will be
          aggregated.
      time_step_alignment {String}:
          Defines how the cube structure will occur based on a given
          time_step_interval.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          START_TIME alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an END_TIME alignment). If the reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an END_TIME alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to used to align the time-step intervals. If you want to
          bin your data weekly from Monday to Sunday, for example, you could set
          a reference time of Sunday at midnight to ensure bins break between
          Sunday and Monday at midnight.
      variables {Value Table}:
          The numeric field containing attribute values that will be brought
          into the space-time cube. Available fill types are:

          * DROP_LOCATIONS-Locations with missing data for any of the variables
          will be dropped from the output space-time cube.

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors.

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Null values present in any of the variable records will result in an
          empty bin. If there are null values present in your input features, it
          is highly recommended that you run the Fill Missing Values tool first.
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space-time cube. Multiple
          statistic and field combinations can be specified. Null values in any
          of the fields specified will result in that feature being dropped from
          the output cube. If there are null values present in your input
          features, it is highly recommended you run the Fill Missing Values
          tool before creating a space time cube. Available statistic
          types are:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field within each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          within each bin.

          * STD-Finds the standard deviation on values in the specified field
          within each bin.

          * MEDIAN-Finds the sorted middle value of all records of the specified
          field within each bin.
                 Available fill types are:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Null values present in any of the summary field records will result in
          those features being excluded from the output cube. If there are null
          values present in your Input Features, it is highly recommended that
          you run the Fill Missing Values tool first. If, after running the Fill
          Missing Values tool, there are still null values present and having
          the count of points in each bin is part of your analysis strategy, you
          may want to consider creating separate cubes, one for the count
          (without Summary Fields) and one for Summary Fields. If the set of
          null values is different for each summary field, you may also consider
          creating a separate cube for each summary field.
      in_related_table {Table View}:
          The table or table view to be related to the input features.
      related_location_id {Field}:
          An integer field in the related table that contains the location ID on
          which the relate will be based.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCubeDefinedLocations_stpm(*gp_fixargs((in_features, output_cube, location_id, temporal_aggregation, time_field, time_step_interval, time_step_alignment, reference_time, variables, summary_fields, in_related_table, related_location_id), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateSpaceTimeCubeMDRasterLayer_stpm', None)
def CreateSpaceTimeCubeMDRasterLayer(in_md_raster=None, output_cube=None, fill_empty_bins=None):
    """CreateSpaceTimeCubeMDRasterLayer_stpm(in_md_raster, output_cube, {fill_empty_bins})

        Creates a space-time cube from a multidimensional raster layer and
        structures the data into space-time bins for efficient space-time
        analysis and visualization.

     INPUTS:
      in_md_raster (Raster Layer):
          The input multidimensional raster layer that will be converted into a
          space-time cube.
      fill_empty_bins {String}:
          Specifies how missing values in the output space-time cube will be
          filled. Each space-time bin in the output must have a value, so you
          must choose how to fill in values for raster cells with NoData values.

          * ZEROS-Empty bins with be filled with zeros. This is the default.

          * SPATIAL_NEIGHBORS-Empty bins will be filled with the average value
          of spatial neighbors.

          * SPACE_TIME_NEIGHBORS-Empty bins will be filled with the average
          value of space-time neighbors.

          * TEMPORAL_TREND-Empty bins will be filled using an interpolated
          univariate spline algorithm.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCubeMDRasterLayer_stpm(*gp_fixargs((in_md_raster, output_cube, fill_empty_bins), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('EmergingHotSpotAnalysis_stpm', None)
def EmergingHotSpotAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, polygon_mask=None, conceptualization_of_spatial_relationships=None, number_of_neighbors=None, define_global_window=None):
    """EmergingHotSpotAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {polygon_mask}, {conceptualization_of_spatial_relationships}, {number_of_neighbors}, {define_global_window})

        Identifies trends in the clustering of point densities (counts) or
        values in a space-time cube created using either the Create Space Time
        Cube By Aggregating Points, Create Space Time Cube From Defined
        Locations or Create Space Time Cube from Multidimensional Raster Layer
        tool. Categories include new, consecutive, intensifying, persistent,
        diminishing, sporadic, oscillating, and historical hot and cold spots.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc) extension
          and must have been created using the Create Space Time Cube By
          Aggregating Points tool or the Create Space Time Cube From Defined
          Locations tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value determines
          which features are analyzed together in order to assess local space-
          time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the
          in_cube that fall outside of the mask will not be included in the
          analysis.This parameter is only available for grid cubes.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among bins are defined.

          * FIXED_DISTANCE-Each bin is analyzed within the context of
          neighboring bins. Neighboring bins inside the specified critical
          distance (neighborhood_distance) receive a weight of one and exert
          influence on computations for the target bin. Neighboring bins outside
          the critical distance receive a weight of zero and have no influence
          on a target bin's computations.

          * K_NEAREST_NEIGHBORS-The closest k bins are included in the analysis
          for the target bin; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring bins that share an edge will
          influence computations for the target polygon bin.

          * CONTIGUITY_EDGES_CORNERS-Bins that share an edge or share a node
          will influence computations for the target polygon bin.
      number_of_neighbors {Long}:
          An integer specifying either the minimum or the exact number of
          neighbors to include in calculations for the target bin. For
          K_NEAREST_NEIGHBORS, each bin will have exactly this specified number
          of neighbors. For FIXED_DISTANCE_BAND, each bin will have at least
          this many neighbors (the threshold distance will be temporarily
          extended to ensure this many neighbors if necessary). When one of the
          contiguity conceptualizations are selected, each bin will be assigned
          this minimum number of neighbors. For bins with fewer than this number
          of contiguous neighbors, additional neighbors will be based on feature
          centroid proximity.
      define_global_window {String}:
          The statistic works by comparing a local statistic calculated from the
          neighbors for each bin to a global value. This parameter can be used
          to control which bins are used to calculate the global value.

          * ENTIRE_CUBE-Each neighborhood is analyzed in comparison to the
          entire cube. This is the default.

          * NEIGHBORHOOD_TIME_STEP-Each neighborhood is analyzed in comparison
          to the bins contained within the Neighborhood Time Step specified.

          * INDIVIDUAL_TIME_STEP-Each neighborhood is analyzed in comparison to
          the bins in the same time step.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the hot and cold spot trends in your
          data. It will show, for example, any new or intensifying hot spots."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EmergingHotSpotAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, polygon_mask, conceptualization_of_spatial_relationships, number_of_neighbors, define_global_window), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('LocalOutlierAnalysis_stpm', None)
def LocalOutlierAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, number_of_permutations=None, polygon_mask=None, conceptualization_of_spatial_relationships=None, number_of_neighbors=None, define_global_window=None):
    """LocalOutlierAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {number_of_permutations}, {polygon_mask}, {conceptualization_of_spatial_relationships}, {number_of_neighbors}, {define_global_window})

        Identifies statistically significant clusters and outliers in the
        context of both space and time. This tool is a space-time
        implementation of the Anselin Local Moran's I statistic.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc) extension
          and must have been created using either the Create Space Time Cube By
          Aggregating Points or Create Space Time Cube From Defined Features
          tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value determines
          which features are analyzed together in order to assess local space-
          time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      number_of_permutations {Long}:
          The number of random permutations for the calculation of pseudo
          p-values. The default number of permutations is 499. If you choose 0
          permutations, the standard p-value is calculated.

          * 0-Permutations are not used and a standard p-value is calculated.

          * 99-With 99 permutations, the smallest possible pseudo p-value is
          0.01 and all other pseudo p-values will be even multiples of this
          value.

          * 199-With 199 permutations, the smallest possible pseudo p-value is
          0.005 and all other pseudo p-values will be even multiples of this
          value.

          * 499-With 499 permutations, the smallest possible pseudo p-value is
          0.002 and all other pseudo p-values will be even multiples of this
          value.

          * 999-With 999 permutations, the smallest possible pseudo p-value is
          0.001 and all other pseudo p-values will be even multiples of this
          value.

          * 9999-With 9999 permutations, the smallest possible pseudo p-value is
          0.0001 and all other pseudo p-values will be even multiples of this
          value.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the
          in_cube that fall outside of the mask will not be included in the
          analysis.This parameter is only available for grid cubes.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among bins are defined.

          * FIXED_DISTANCE-Each bin is analyzed within the context of
          neighboring bins. Neighboring bins inside the specified critical
          distance (neighborhood_distance) receive a weight of one and exert
          influence on computations for the target bin. Neighboring bins outside
          the critical distance receive a weight of zero and have no influence
          on a target bin's computations.

          * K_NEAREST_NEIGHBORS-The closest k bins are included in the analysis
          for the target bin; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring bins that share an edge will
          influence computations for the target polygon bin.

          * CONTIGUITY_EDGES_CORNERS-Bins that share an edge or share a node
          will influence computations for the target polygon bin.
      number_of_neighbors {Long}:
          An integer specifying either the minimum or the exact number of
          neighbors to include in calculations for the target bin. For
          K_NEAREST_NEIGHBORS, each bin will have exactly this specified number
          of neighbors. For FIXED_DISTANCE, each bin will have at least this
          many neighbors (the neighborhood_distance will be temporarily extended
          to ensure this many neighbors if necessary). When one of the
          contiguity conceptualizations are selected, each bin will be assigned
          this minimum number of neighbors. For bins with fewer than this number
          of contiguous neighbors, additional neighbors will be based on feature
          centroid proximity.
      define_global_window {String}:
          The Anselin Local Moran's I statistic works by comparing a local
          statistic calculated from the neighbors for each bin to a global
          value. This parameter can be used to control which bins are used to
          calculate the global value.

          * ENTIRE_CUBE-Each neighborhood is analyzed in comparison to the
          entire cube. This is the default.

          * NEIGHBORHOOD_TIME_STEP-Each neighborhood is analyzed in comparison
          to the bins contained within the specified Neighborhood Time Step.

          * INDIVIDUAL_TIME_STEP-Each neighborhood is analyzed in comparison to
          the bins in the same time step.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class containing locations that were considered
          statistically significant clusters or outliers."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.LocalOutlierAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, number_of_permutations, polygon_mask, conceptualization_of_spatial_relationships, number_of_neighbors, define_global_window), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('TimeSeriesClustering_stpm', None)
def TimeSeriesClustering(in_cube=None, analysis_variable=None, output_features=None, characteristic_of_interest=None, cluster_count=None, output_table_for_charts=None, shape_characteristic_to_ignore=None, enable_time_series_popups=None):
    """TimeSeriesClustering_stpm(in_cube, analysis_variable, output_features, characteristic_of_interest, {cluster_count}, {output_table_for_charts}, {shape_characteristic_to_ignore;shape_characteristic_to_ignore...}, {enable_time_series_popups})

        Partitions a collection of time series, stored in a space-time cube,
        based on the similarity of time series characteristics. Time series
        can be clustered based on three criteria: having similar values across
        time, tending to increase and decrease at the same time, and having
        similar repeating patterns. The output of this tool is a 2D map
        displaying each location in the cube symbolized by cluster membership
        and messages. The output also includes charts containing information
        about the representative time series signature for each cluster.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an .nc extension
          and must have been created using the Create Space Time Cube By
          Aggregating Points, Create Space Time Cube From Defined Features, or
          Create Space Time Cube From Multidimensional Raster Layer tool.
      analysis_variable (String):
          The numeric variable in the netCDF file, changing over time, that will
          be used to distinguish one cluster from another.
      characteristic_of_interest (String):
          Specifies the characteristic of the time series that will be used to
          determine which locations should be clustered together.

          * VALUE-Locations with similar values across time will be clustered
          together.

          * PROFILE-Locations with values that tend to increase and decrease
          proportionally at the same times will be clustered together.

          * PROFILE_FOURIER-Locations with values that have similar smooth,
          periodic patterns will be clustered together.
      cluster_count {Long}:
          The number of clusters to create. When left empty, the tool will
          evaluate the optimal number of clusters using a pseudo-F statistic.
          The optimal number of clusters will be reported in the messages
          window.
      shape_characteristic_to_ignore {String}:
          Specifies characteristics that will be ignored when determining the
          similarity between two time series.

          * TIME_LAG-The starting time of each period, including time lags,
          will be ignored. For example, if two time series have similar periodic
          patterns, but the values of one are three days behind the other, the
          time series will be considered similar.

          * RANGE-The magnitude of the values in each period will be ignored.
          For example, if two time series begin and end their periods at the
          same times, they will be considered similar, even if the actual values
          are very different.
          If both characteristics are ignored, two time series will be
          considered similar if the durations of the periods are similar, even
          if they start at different times and have different values within the
          periods.
      enable_time_series_popups {Boolean}:
          Specifies whether time series charts will be created in the pop-ups of
          each output feature showing the time series of the feature and the
          average time series of all features in the same cluster as the
          feature. Time series pop-ups are not supported for shapefile outputs.

          * CREATE_POPUP-Time series charts will be created for the output
          features.

          * NO_POPUP-Time series charts will not be created. This is the
          default.

     OUTPUTS:
      output_features (Feature Class):
          The new output feature class containing all locations in the space-
          time cube and a field indicating cluster membership. This feature
          class will be a two-dimensional representation of the clusters in your
          data.
      output_table_for_charts {Table}:
          If specified, this table contains the representative time series for
          each cluster based on both the average for each time series cluster
          and the medoid time series. Charts created from this table can be
          accessed in the Standalone Tables section."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.TimeSeriesClustering_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, characteristic_of_interest, cluster_count, output_table_for_charts, shape_characteristic_to_ignore, enable_time_series_popups), True)))
        return retval
    except Exception as e:
        raise e


# Time Series Forecasting toolset
@gptooldoc('CurveFitForecast_stpm', None)
def CurveFitForecast(in_cube=None, analysis_variable=None, output_features=None, output_cube=None, number_of_time_steps_to_forecast=None, curve_type=None, number_for_validation=None):
    """CurveFitForecast_stpm(in_cube, analysis_variable, output_features, {output_cube}, {number_of_time_steps_to_forecast}, {curve_type}, {number_for_validation})

        Forecasts the future values of each location of a space-time cube
        using curve fitting.The tool fits a parametric curve to each location
        in the Input Space
        Time Cube and forecasts the time series by extrapolating this curve to
        future time steps. The curves can be linear, parabolic, S-shaped
        (Gompertz), or exponential. You can use the same curve type at each
        location of the space-time cube or allow the tool to set which curve
        type best fits each location.

     INPUTS:
      in_cube (File):
          The netCDF cube containing the variable you want to forecast to future
          time steps. This file must have an .nc file extension and must have
          been created using the Create Space Time Cube By Aggregating Points,
          Create Space Time Cube From Defined Locations, or Create Space Time
          Cube From Multidimensional Raster Layer tools.
      analysis_variable (String):
          The numeric variable in the netCDF file that will be forecasted to
          future time steps.
      number_of_time_steps_to_forecast {Long}:
          A positive integer specifying the number of time steps to forecast.
          This value cannot be larger than 50 percent of the total time steps in
          the input space-time cube. The default value is one time step.
      curve_type {String}:
          Specifies the curve type that will be used to forecast the values of
          the input space-time cube.

          * LINEAR-The time series increases or decreases linearly over time.

          * PARABOLIC-The time series follows a parabola or quadratic curve over
          time.

          * EXPONENTIAL-The time series increases or decreases exponentially
          over time.

          * GOMPERTZ-The time series increases or decreases following the shape
          of an S over time.

          * AUTO_DETECT-All four curve types are run for each location and the
          model is provided the smallest Validation RMSE. If no time slices are
          excluded for validation, the model with the smallest Forecast RMSE is
          used. This is the default.
      number_for_validation {Long}:
          The number of time steps at the end of each time series to exclude for
          validation. The default value is 10 percent (rounded down) of the
          number of input time steps, and this value cannot be larger than 25
          percent of the number of time steps. Provide the value 0 to not
          exclude any time steps.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class of all locations in the space-time cube with
          forecasted values stored as fields. The layer displays the forecast
          for the final time step and contains pop-ups charts showing the time
          series and forecasts for each location.
      output_cube {File}:
          A new space-time cube (.nc file) containing the values of the input
          space-time cube with the forecasted time steps appended. The Visualize
          Space Time Cube in 3D tool can be used to see all of the observed and
          forecasted values simultaneously."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CurveFitForecast_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, output_cube, number_of_time_steps_to_forecast, curve_type, number_for_validation), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('EvaluateForecastsByLocation_stpm', None)
def EvaluateForecastsByLocation(in_cubes=None, output_features=None, output_cube=None, evaluate_using_validation_results=None):
    """EvaluateForecastsByLocation_stpm(in_cubes;in_cubes..., output_features, {output_cube}, {evaluate_using_validation_results})

        Selects the most accurate among multiple forecasting results for each
        location of a space-time cube. This allows you to use multiple tools
        in the Time Series Forecasting toolset with the same time series data
        and select the best forecast for each location.

     INPUTS:
      in_cubes (File):
          The input space-time cubes containing forecasts to be compared. To be
          compared, all forecast cubes must be created from the same original
          time series data.
      evaluate_using_validation_results {Boolean}:
          Specifies whether the forecast method for a location will be
          determined using the smallest Validation RMSE or smallest Forecast
          RMSE.

          * USE_VALIDATION-The forecast method will be determined using the
          smallest Validation RMSE. This is the default.

          * NO_VALIDATION-The forecast method will be determined using the
          smallest Forecast RMSE.

     OUTPUTS:
      output_features (Feature Class):
          The new output feature class representing the locations of the space-
          time cube and fields containing the forecast values of the selected
          method at each location. The pop-ups of the features display charts of
          the original time series data and the forecasts of all methods.
      output_cube {File}:
          The output space-time cube (.nc file) containing the original time
          series data with the forecasts of the selected method at each
          location. The Visualize Space Time Cube in 3D tool can be used to view
          the original and forecasted values simultaneously."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EvaluateForecastsByLocation_stpm(*gp_fixargs((in_cubes, output_features, output_cube, evaluate_using_validation_results), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ExponentialSmoothingForecast_stpm', None)
def ExponentialSmoothingForecast(in_cube=None, analysis_variable=None, output_features=None, output_cube=None, number_of_time_steps_to_forecast=None, season_length=None, number_for_validation=None):
    """ExponentialSmoothingForecast_stpm(in_cube, analysis_variable, output_features, {output_cube}, {number_of_time_steps_to_forecast}, {season_length}, {number_for_validation})

        Forecasts the future values of each location of a space-time cube
        using the Holt-Winters exponential smoothing method by decomposing the
        time series at each location cube into seasonal and trend components

     INPUTS:
      in_cube (File):
          The netCDF cube containing the variable you want to forecast to future
          time steps. This file must have an .nc file extension and must have
          been created using the Create Space Time Cube By Aggregating Points,
          Create Space Time Cube From Defined Locations, or Create Space Time
          Cube From Multidimensional Raster Layer tools.
      analysis_variable (String):
          The numeric variable in the netCDF file that will be forecasted to
          future time steps.
      number_of_time_steps_to_forecast {Long}:
          A positive integer specifying the number of time steps to forecast.
          This value cannot be larger than 50 percent of the total time steps in
          the input space-time cube. The default value is one time step.
      season_length {Long}:
          The number of time steps corresponding to one season at each location.
          If there are multiple seasons in your data, it is recommended that you
          use the longest season to produce a most reliable result. If left
          empty, a season length will be estimated by the tool at each location
          using a spectral density function.
      number_for_validation {Long}:
          The number of time steps at the end of each time series to exclude for
          validation. The default value is 10 percent (rounded down) of the
          number of input time steps, and this value cannot be larger than 25
          percent of the number of time steps. Provide the value 0 to not
          exclude any time steps.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class of all locations in the space-time cube with
          forecasted values stored as fields. The layer displays the forecast
          for the final time step and contains pop-up charts showing the time
          series, forecasts, and 90 percent confidence bounds for each location.
      output_cube {File}:
          A new space-time cube (.nc file) containing the values of the input
          space-time cube with the forecasted time steps appended. The Visualize
          Space Time Cube in 3D tool can be used to see all of the observed and
          forecasted values simultaneously."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ExponentialSmoothingForecast_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, output_cube, number_of_time_steps_to_forecast, season_length, number_for_validation), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ForestBasedForecast_stpm', None)
def ForestBasedForecast(in_cube=None, analysis_variable=None, output_features=None, output_cube=None, number_of_time_steps_to_forecast=None, time_window=None, number_for_validation=None, number_of_trees=None, minimum_leaf_size=None, maximum_depth=None, sample_size=None, forecast_approach=None):
    """ForestBasedForecast_stpm(in_cube, analysis_variable, output_features, {output_cube}, {number_of_time_steps_to_forecast}, {time_window}, {number_for_validation}, {number_of_trees}, {minimum_leaf_size}, {maximum_depth}, {sample_size}, {forecast_approach})

        Forecasts the future values of each location of a space-time cube
        using an adaptation of Leo Breiman's random forest algorithm. The
        forest regression model is trained using time windows on each location
        of the space-time cube.

     INPUTS:
      in_cube (File):
          The netCDF cube containing the variable you want to forecast to future
          time steps. This file must have an .nc file extension and must have
          been created using the Create Space Time Cube By Aggregating Points,
          Create Space Time Cube From Defined Locations, or Create Space Time
          Cube From Multidimensional Raster Layer tools.
      analysis_variable (String):
          The numeric variable in the netCDF file that will be forecasted to
          future time steps.
      number_of_time_steps_to_forecast {Long}:
          A positive integer specifying the number of time steps to forecast.
          This value cannot be larger than 50 percent of the total time steps in
          the input space-time cube. The default value is one time step.
      time_window {Long}:
          The number of previous time steps to use when training the forest. If
          your data displays seasonality (repeating cycles), provide the number
          of time steps corresponding to one season for this parameter. This
          value cannot be larger than one-third of the number of time steps in
          the input space-time cube. If left empty, a time window is estimated
          for each location using a spectral density function.
      number_for_validation {Long}:
          The number of time steps at the end of each time series to exclude for
          validation. The default value is 10 percent (rounded down) of the
          number of input time steps, and this value cannot be larger than 25
          percent of the number of time steps. Provide the value 0 to not
          exclude any time steps.
      number_of_trees {Long}:
          The number of trees to create in the forest model. More trees will
          generally result in more accurate model prediction, but the model will
          take longer to calculate. The default number of trees is 100, and the
          value must be at least 1 and not greater than 1,000.
      minimum_leaf_size {Long}:
          The minimum number of observations that are required to keep a leaf
          (the terminal node on a tree without further splits). For very large
          data, increasing this number will decrease the run time of the tool.
      maximum_depth {Long}:
          The maximum number of splits that will be made down a tree. Using a
          large maximum depth, more splits will be created, which may increase
          the chances of overfitting the model. If left empty, a value will be
          determined by the tool based on the number of trees created by the
          model and the size of the time step window.
      sample_size {Long}:
          The percent of training data that will be used to fit the forecast
          model. The training data consists of associated explanatory and
          dependent variables constructed using time windows. All remaining
          training data will be used to optimize the parameters of the forecast
          model. The default is 100 percent.
      forecast_approach {String}:
          Specifies how the explanatory and dependent variables will be
          represented when training the forest model at each location.To train
          the forest that will be used to forecast, sets of explanatory
          and dependent variables must be created using time windows. Use this
          parameter to specify whether these variables will be linearly
          detrended and whether the dependent variable will be represented by
          its raw value or by the residual of a linear regression model. This
          linear regression model uses all time steps within a time window as
          explanatory variables and uses the following time step as the
          dependent variable. The residual is calculated by subtracting the
          predicted value based on linear regression from the raw value of the
          dependent variable.

          * VALUE-Values within the time window are not detrended and the
          dependent variable will be represented by its raw value.

          * VALUE_DETREND-Values within the time window are linearly detrended,
          and the dependent variable will be represented by its detrended value.
          This is the default.

          * RESIDUAL-Values within the time window are not detrended, and the
          dependent variable will be represented by the residual of a linear
          regression model using the values within the time window as
          explanatory variables.

          * RESIDUAL_DETREND-Values within the time window are linearly
          detrended, and the dependent variable will be represented by the
          residual of a linear regression model using the detrended values
          within the time window as explanatory variables.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class of all locations in the space-time cube with
          forecasted values stored as fields. The layer displays the forecast
          for the final time step and contains pop-up charts showing the time
          series, forecasts, and 90 percent confidence bounds for each location.
      output_cube {File}:
          A new space-time cube (.nc file) containing the values of the input
          space-time cube with the forecasted time steps appended. The Visualize
          Space Time Cube in 3D tool can be used to see all of the observed and
          forecasted values simultaneously."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ForestBasedForecast_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, output_cube, number_of_time_steps_to_forecast, time_window, number_for_validation, number_of_trees, minimum_leaf_size, maximum_depth, sample_size, forecast_approach), True)))
        return retval
    except Exception as e:
        raise e


# Utilities toolset
@gptooldoc('FillMissingValues_stpm', None)
def FillMissingValues(in_features=None, out_features=None, fields_to_fill=None, fill_method=None, conceptualization_of_spatial_relationships=None, distance_band=None, temporal_neighborhood=None, time_field=None, number_of_spatial_neighbors=None, location_id=None, related_table=None, related_location_id=None, weights_matrix_file=None, unique_id=None, null_value=None, out_table=None):
    """FillMissingValues_stpm(in_features, out_features, fields_to_fill;fields_to_fill..., fill_method, {conceptualization_of_spatial_relationships}, {distance_band}, {temporal_neighborhood}, {time_field}, {number_of_spatial_neighbors}, {location_id}, {related_table}, {related_location_id}, {weights_matrix_file}, {unique_id}, {null_value}, {out_table})

        Replaces missing (null) values with estimated values based on spatial
        neighbors, space-time neighbors, or time-series values.

     INPUTS:
      in_features (Feature Layer):
          The feature class containing the null values to be filled.
      fields_to_fill (Field):
          The numeric fields containing missing data (null values).
      fill_method (String):
          The type of calculation that will be applied. TEMPORAL_TREND is only
          available if location_id and time_field are specified.

          * AVERAGE-Replaces null values with the average (mean) value of the
          feature's neighbors.

          * MINIMUM-Replaces null values with the minimum (smallest) value of
          the feature's neighbors.

          * MAXIMUM-Replaces null values with the maximum (largest) value of the
          feature's neighbors.

          * MEDIAN-Replaces null values with the median (sorted middle value) of
          the feature's neighbors.

          * TEMPORAL_TREND-Replaces null values based on the trend at that
          unique location.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among features are defined.

          * FIXED_DISTANCE-Neighboring features within a specified critical
          distance (distance_band) of each feature are included in the
          calculations; everything outside the critical distance is excluded.

          * K_NEAREST_NEIGHBORS-The closest k features are included in the
          calculations; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring polygon features that share
          a boundary or overlap will influence computations for the target
          polygon feature.

          * CONTIGUITY_EDGES_CORNERS-Polygon features that share a boundary,
          share a node, or overlap will influence computations for the target
          polygon feature.

          * GET_SPATIAL_WEIGHTS_FROM_FILE-Spatial relationships are defined by a
          specified spatial weights file. The path to the spatial weights file
          is specified by the Weights_Matrix_File parameter.
      distance_band {Linear Unit}:
          Specifies a cutoff distance for the FIXED_DISTANCE option. Features
          outside the specified cutoff for a target feature are ignored in
          calculations for that feature. This parameter is not available for
          CONTIGUITY_EDGES_ONLY or CONTIGUITY_EDGES_CORNERS.
      temporal_neighborhood {Time Unit}:
          Specifies an interval forward and backward in time to determine which
          features will be used in calculations for the target feature. Features
          that are not within this interval of the target feature are ignored in
          calculations for that feature.
      time_field {Field}:
          The field containing the time stamp for each record in the dataset.
          This field must be of type Date.This is required if location_id has
          been provided.
      number_of_spatial_neighbors {Long}:
          The number of nearest neighbors to be included in calculations.If
          FIXED_DISTANCE, CONTIGUITY_EDGES_ONLY, or CONTIGUITY_EDGES_CORNERS
          is chosen, this number is the minimum number of neighbors to include
          in calculations.
      location_id {Field}:
          An integer field containing a unique ID number for each
          location.location_id is used to match features from the in_features to
          rows in
          the related_table or to specify a unique location ID for determining
          temporal neighbors.
      related_table {Table View}:
          The table or table view containing the temporal data for each of the
          in_features.
      related_location_id {Field}:
          An integer field in the related_table that contains the location_id on
          which the relate will be based.
      weights_matrix_file {File}:
          The path to a file containing weights that define spatial, and
          potentially temporal, relationships among features.
      unique_id {Field}:
          An integer field containing a different value for every record in the
          in_features. This field can be used to join your results back to your
          original dataset.If you don't have a unique_id field, you can create
          one by adding an
          integer field to your feature class table and calculating the field
          values equal to the FID or OBJECTID field.
      null_value {Double}:
          The value that represents null (missing) values. If no value is
          specified, <Null> is assumed for geodatabase feature classes. For
          shapefile input, a numeric value of the null placeholder is required.

     OUTPUTS:
      out_features (Feature Class):
          The output that will include the filled (estimated) values.If
          related_table is specified, out_features will contain the number of
          estimated values at each location, and out_table will contain the
          filled (estimated) values.
      out_table {Table}:
          The output table containing the filled (estimated) values.The output
          table is required if a related table is entered."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FillMissingValues_stpm(*gp_fixargs((in_features, out_features, fields_to_fill, fill_method, conceptualization_of_spatial_relationships, distance_band, temporal_neighborhood, time_field, number_of_spatial_neighbors, location_id, related_table, related_location_id, weights_matrix_file, unique_id, null_value, out_table), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('VisualizeSpaceTimeCube2D_stpm', None)
def VisualizeSpaceTimeCube2D(in_cube=None, cube_variable=None, display_theme=None, output_features=None, enable_time_series_popups=None):
    """VisualizeSpaceTimeCube2D_stpm(in_cube, cube_variable, display_theme, output_features, {enable_time_series_popups})

        Visualizes the variables stored in a netCDF cube and the results
        generated by the Space Time Pattern Mining tools. Output from this
        tool is a two-dimensional representation uniquely rendered based on
        the variable and theme specified.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using either the
          Create Space Time Cube By Aggregating Points or Create Space Time Cube
          From Defined Locations tool.
      cube_variable (String):
          The numeric variable in the netCDF cube to explore. The cube will
          always contain the COUNT variable. Any Summary Fields or Variables
          will also be available if they were included when the cube was
          created.
      display_theme (String):
          Specifies the characteristic of the Cube Variable to display. Options
          will vary depending on how the cube was created and the analyses that
          were run.If the cube was created by aggregating points, Locations with
          data and
          Trends will always be available. Number of estimated bins and
          Locations excluded from analysis will only be available for those
          Summary Fields that were included in the cube creation process.If the
          cube was created from defined locations, Trends will be
          available for those Summary Fields or Variables that were included in
          the cube creation process.Hot and cold spot trends and Emerging Hot
          Spot Analysis results will
          only be available after Emerging Hot Spot Analysis has been run on the
          selected Cube Variable. Percentage of local outliers, Local outlier in
          the most recent time period, Local Outlier Analysis results, and
          Locations without spatial neighbors are only available when the Local
          Outlier Analysis tool has been run.Forecast results will only be
          available for cubes created by tools in
          the Time Series Forecasting toolset.

          * LOCATIONS_WITH_DATA-All locations that contain data for the chosen
          variable are displayed.

          * TRENDS-The trend of values at each location (determined using the
          Mann-Kendall statistic) are displayed.

          * HOT_AND_COLD_SPOT_TRENDS-The trend of z-scores at each location
          (determined using the Mann-Kendall statistic) are displayed.

          * EMERGING_HOT_SPOT_ANALYSIS_RESULTS-The results of Emerging Hot Spot
          Analysis for the specified Cube Variable are displayed.

          * LOCAL_OUTLIER_ANALYSIS_RESULTS-The results of Local Outlier Analysis
          for the specified Cube Variable are displayed.

          * PERCENTAGE_OF_LOCAL_OUTLIERS-The total percentage of outliers at
          each location are displayed.

          * LOCAL_OUTLIER_IN_MOST_RECENT_TIME_PERIOD-All outliers that are
          present in the most recent time period are displayed.

          * TIME_SERIES_CLUSTERING_RESULTS-The results of Time Series Clustering
          for the specified Cube Variable are displayed.

          * LOCATIONS_WITHOUT_SPATIAL_NEIGHBORS-For the last analysis run, all
          locations that have no spatial neighbors and, as a result, are relying
          on temporal neighbors for analysis are displayed.

          * NUMBER_OF_ESTIMATED_BINS-The number of bins that were estimated for
          each location are displayed

          * LOCATIONS_EXCLUDED_FROM_ANALYSIS-The locations that were excluded
          from analysis because they had empty bins that did not meet the
          criteria for estimation are displayed.

          * FORECAST_RESULTS-The results of the tool from the Time Series
          Forecasting toolset for the specified Analysis Variable are displayed.
      enable_time_series_popups {Boolean}:
          Specifies whether time series pop-ups will be generated for each
          output feature. Pop-up charts are not supported for shapefile outputs.

          * CREATE_POPUP-Time series pop-ups will be generated for each feature
          in the dataset.

          * NO_POPUP-Time series pop-ups will not be generated. This is the
          default.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the specified display variable."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube2D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features, enable_time_series_popups), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('VisualizeSpaceTimeCube3D_stpm', None)
def VisualizeSpaceTimeCube3D(in_cube=None, cube_variable=None, display_theme=None, output_features=None):
    """VisualizeSpaceTimeCube3D_stpm(in_cube, cube_variable, display_theme, output_features)

        Visualizes the variables stored in a netCDF cube created with the
        Space Time Pattern Mining tools. Output from this tool is a three-
        dimensional representation uniquely rendered based on the variable and
        theme chosen.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using either the
          Create Space Time Cube By Aggregating Points or Create Space Time Cube
          From Defined Locations tool.
      cube_variable (String):
          The numeric variable in the netCDF cube that you want to explore. The
          cube will always contain the COUNT variable if aggregation was used in
          cube creation. Any summary fields or variables will also be available
          if they were included in the cube creation process.
      display_theme (String):
          Specifies the characteristic of the Cube Variable parameter to be
          displayed. Available options will vary depending on how the cube was
          created and the analyses run.

          * VALUE-The numeric value of the cube variable chosen will be
          displayed.

          * HOT_AND_COLD_SPOT_RESULTS-The statistical significance of each bin
          based on the space-time hot spot analysis run in Emerging Hot Spot
          Analysis will be displayed.

          * ESTIMATED_BINS-Bins with estimated values will be displayed.

          * LOCAL_OUTLIER_RESULTS-The cluster or outlier type (COType) assigned
          to each bin from the analysis run in Local Outlier Analysis will be
          displayed.

          * TEMPORAL_AGGREGATION_COUNT-The count of records aggregated into each
          space-time bin will be displayed.

          * FORECAST_RESULTS-The input time steps and forecasted values from
          time series forecasting will be displayed.
          Value is the numeric value of the Cube Variable parameter and is
          always available. Estimated bins are only available for the summary
          fields that were included in the cube creation process. Hot and cold
          spot results will only be available for the Cube Variable parameter
          value for which Emerging Hot Spot Analysis has been run. Cluster and
          outlier results will only be available for Cube Variables for which
          Local Outlier Analysis has been run. Temporal aggregation count will
          only be available for defined location cubes that have been aggregated
          temporally. Forecast results will only be available for the Cube
          Variable parameter value for which a tool in the Time Series
          Forecasting toolset has been run.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a three-
          dimensional map representation of the display variable that can be
          displayed in a 3D scene."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube3D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject